@@ -33,6 +33,7 @@
 #include <QTextLayout>
 
 class ChatLine;
+class ChatView;
 
 /* All external positions are relative to the parent ChatLine */
 /* Yes, that's also true for the boundingRect() and related things */
@@ -41,12 +42,13 @@ class ChatItem {
 protected:
   // boundingRect is relative to the parent ChatLine
   ChatItem(const QRectF &boundingRect, ChatLine *parent);
-  virtual ~ChatItem() {}
+  virtual ~ChatItem();
 
 public:
   const QAbstractItemModel *model() const;
   ChatLine *chatLine() const;
   ChatScene *chatScene() const;
+  ChatView *chatView() const;
   int row() const;
   virtual ChatLineModel::ColumnType column() const = 0;
 
@@ -63,14 +65,6 @@ public:
   QPointF mapToScene(const QPointF &) const;
   QPointF mapFromScene(const QPointF &) const;
 
-  void initLayoutHelper(QTextLayout *layout, QTextOption::WrapMode, Qt::Alignment = Qt::AlignLeft) const;
-  virtual inline void initLayout(QTextLayout *layout) const {
-    initLayoutHelper(layout, QTextOption::NoWrap);
-    doLayout(layout);
-  }
-  virtual void doLayout(QTextLayout *) const;
-  virtual UiStyle::FormatList formatList() const;
-
   virtual void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
   virtual inline int type() const { return ChatScene::ChatItemType; }
 
@@ -89,6 +83,13 @@ public:
   virtual void addActionsToMenu(QMenu *menu, const QPointF &itemPos);
   virtual void handleClick(const QPointF &pos, ChatScene::ClickMode);
 
+  void initLayoutHelper(QTextLayout *layout, QTextOption::WrapMode, Qt::Alignment = Qt::AlignLeft) const;
+
+  //! Remove internally cached data
+  /** This removes e.g. the cached QTextLayout to avoid wasting space for nonvisible ChatLines
+   */
+  virtual void clearCache();
+
 protected:
   enum SelectionMode {
     NoSelection,
@@ -99,9 +100,15 @@ protected:
   virtual void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
   virtual void mousePressEvent(QGraphicsSceneMouseEvent *event);
   virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
-  virtual void hoverEnterEvent(QGraphicsSceneHoverEvent *) {};
-  virtual void hoverLeaveEvent(QGraphicsSceneHoverEvent *) {};
-  virtual void hoverMoveEvent(QGraphicsSceneHoverEvent *) {};
+  virtual void hoverEnterEvent(QGraphicsSceneHoverEvent *) {}
+  virtual void hoverLeaveEvent(QGraphicsSceneHoverEvent *) {}
+  virtual void hoverMoveEvent(QGraphicsSceneHoverEvent *) {}
+
+  QTextLayout *layout() const;
+
+  virtual void initLayout(QTextLayout *layout) const;
+  virtual void doLayout(QTextLayout *) const;
+  virtual UiStyle::FormatList formatList() const;
 
   void paintBackground(QPainter *);
   QVector<QTextLayout::FormatRange> selectionFormats() const;
@@ -118,9 +125,9 @@ protected:
 
   qint16 posToCursor(const QPointF &pos) const;
 
-  inline void setGeometry(qreal width, qreal height) { _boundingRect.setSize(QSizeF(width, height)); }
-  inline void setHeight(const qreal &height) { _boundingRect.setHeight(height); }
-  inline void setWidth(const qreal &width) { _boundingRect.setWidth(width); }
+  inline void setGeometry(qreal width, qreal height) { clearCache(); _boundingRect.setSize(QSizeF(width, height)); }
+  inline void setHeight(const qreal &height) { clearCache(); _boundingRect.setHeight(height); }
+  inline void setWidth(const qreal &width) { clearCache(); _boundingRect.setWidth(width); }
   inline void setPos(const QPointF &pos) { _boundingRect.moveTopLeft(pos); }
 
 private:
@@ -130,6 +137,8 @@ private:
   SelectionMode _selectionMode;
   qint16 _selectionStart, _selectionEnd;
 
+  mutable QTextLayout *_cachedLayout;
+
   // internal selection stuff
   void setSelection(int start, int length);
 
@@ -161,10 +170,7 @@ public:
 protected:
   virtual void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
   virtual inline int type() const { return ChatScene::SenderChatItemType; }
-  virtual inline void initLayout(QTextLayout *layout) const {
-    initLayoutHelper(layout, QTextOption::ManualWrap, Qt::AlignRight);
-    doLayout(layout);
-  }
+  virtual void initLayout(QTextLayout *layout) const;
 };
 
 // ************************************************************
@@ -185,6 +191,8 @@ public:
   inline ChatLineModel::ColumnType column() const { return ChatLineModel::ContentsColumn; }
   QFontMetricsF *fontMetrics() const;
 
+  virtual void clearCache();
+
 protected:
   virtual void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
   virtual void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
@@ -196,10 +204,7 @@ protected:
 
   virtual QVector<QTextLayout::FormatRange> additionalFormats() const;
 
-  virtual inline void initLayout(QTextLayout *layout) const {
-    initLayoutHelper(layout, QTextOption::WrapAtWordBoundaryOrAnywhere);
-    doLayout(layout);
-  }
+  virtual void initLayout(QTextLayout *layout) const;
   virtual void doLayout(QTextLayout *layout) const;
   virtual UiStyle::FormatList formatList() const;
 
@@ -207,7 +212,7 @@ private:
   class ActionProxy;
   class WrapColumnFinder;
 
-  ContentsChatItemPrivate *_data;
+  mutable ContentsChatItemPrivate *_data;
   ContentsChatItemPrivate *privateData() const;
 
   Clickable clickableAt(const QPointF &pos) const;
@@ -217,13 +222,14 @@ private:
   void clearWebPreview();
 
   qreal setGeometryByWidth(qreal w);
-  friend class ChatLine;
-  friend struct ContentsChatItemPrivate;
 
   QFontMetricsF *_fontMetrics;
 
   // we need a receiver for Action signals
   static ActionProxy _actionProxy;
+
+  friend class ChatLine;
+  friend struct ContentsChatItemPrivate;
 };
 
 struct ContentsChatItemPrivate {
