@@ -26,7 +26,6 @@
 #include "client.h"
 #include "iconloader.h"
 #include "ircuser.h"
-#include "jumpkeyhandler.h"
 #include "networkmodel.h"
 #include "qtui.h"
 #include "qtuisettings.h"
@@ -54,7 +53,6 @@ InputWidget::InputWidget(QWidget *parent)
 
   ui.ownNick->setSizeAdjustPolicy(QComboBox::AdjustToContents);
   ui.ownNick->installEventFilter(new MouseWheelFilter(this));
-  ui.inputEdit->installEventFilter(new JumpKeyHandler(this));
   ui.inputEdit->installEventFilter(this);
 
   ui.inputEdit->setMinHeight(1);
@@ -106,6 +104,9 @@ InputWidget::InputWidget(QWidget *parent)
   setEnableSpellCheck(s.value("EnableSpellCheck", false));
 #endif
 
+  s.notify("EnableEmacsMode", this, SLOT(setEnableEmacsMode(QVariant)));
+  setEnableEmacsMode(s.value("EnableEmacsMode", false));
+
   s.notify("ShowNickSelector", this, SLOT(setShowNickSelector(QVariant)));
   setShowNickSelector(s.value("ShowNickSelector", true));
 
@@ -161,6 +162,10 @@ void InputWidget::setEnableSpellCheck(const QVariant &v) {
   ui.inputEdit->setSpellCheckEnabled(v.toBool());
 }
 
+void InputWidget::setEnableEmacsMode(const QVariant &v) {
+  ui.inputEdit->setEmacsMode(v.toBool());
+}
+
 void InputWidget::setShowNickSelector(const QVariant &v) {
   ui.ownNick->setVisible(v.toBool());
 }
@@ -272,7 +277,12 @@ void InputWidget::rowsAboutToBeRemoved(const QModelIndex &parent, int start, int
   }
 }
 
+
 void InputWidget::updateEnabledState() {
+// FIXME: Find a visualization for this that does not disable the widget!
+//        Disabling kills global action shortcuts, plus users sometimes need/want to enter text
+//        even in inactive channels.
+#if 0
   QModelIndex currentIndex = selectionModel()->currentIndex();
 
   const Network *net = Client::networkModel()->networkByIndex(currentIndex);
@@ -283,7 +293,9 @@ void InputWidget::updateEnabledState() {
     // ... if we're not connected to the network at all
     enabled &= net->isConnected();
   }
+
   ui.inputEdit->setEnabled(enabled);
+#endif
 }
 
 const Network *InputWidget::currentNetwork() const {
